package com.jetbyte.cache8;

import java.util.Map;
import java.util.HashMap;
import java.util.LinkedList;

public class EntryUsageTimeoutCache extends TimeLimitedCache
{
   public EntryUsageTimeoutCache(String name)
   {
      super(name, EntryUsageTimeoutCache.class.getName());
   }

   protected Object findObject(Cache.Key key)
   {      
      ObjectRef ref = (ObjectRef)cache.get(key);

      Object obj = null;

      if (ref != null)
      {
         obj = ref.getObject();

         ref.objectUsed();

         lruList.remove(ref);
         lruList.addLast(ref);            
      }

      return obj;
   }

   protected void addObject(Cache.Key key, Object obj)
   {
      ObjectRef ref = new ObjectRef(this, key, obj); 

      cache.put(key, ref);

      lruList.addLast(ref); 
   }

   protected void removeObject(Cache.Key key)
   {
      cache.remove(key);
   }

   synchronized public int entries()
   {
      return cache.size();
   }

   protected void flushCache()
   {
      cache.clear();

      lruList.clear();
   }

   protected long getReapPeriod()
   {
      return scanEveryMillis;
   }

   synchronized protected void reap()
   {
      boolean done = (lruList.size() == 0);
      
      System.out.println("Reaping - " + name());
      System.out.println("LRU size = " + lruList.size());

      while (!done)
      {
         ObjectRef ref = (ObjectRef)lruList.getFirst();

         if (ref.timedOut(entryTimeoutMillis))
         {
            System.out.println("Reference reaped: - " + ref.getKey());

            ref.releaseObject();

            lruList.removeFirst();

            done = (lruList.size() == 0);
         }
         else
         {
            done = true;
         }
      }
   }

   private long entryTimeoutMillis = 30000;  // 30 secs
   
   private long scanEveryMillis = 10000; // 10 secs

   private Map cache = new HashMap(89);  
   
   private LinkedList lruList = new LinkedList(); 

   private static class ObjectRef extends Cache.ObjectRef
   {
      protected ObjectRef(
         Cache cache, 
         Key key, 
         Object obj)
      {
         super(cache, key, obj);

         lastUsedTime = System.currentTimeMillis(); 
      }

      synchronized protected boolean timedOut(long timeoutMillis)
      {
         return (System.currentTimeMillis() - lastUsedTime > timeoutMillis);
      }

      public void objectUsed()
      {
         lastUsedTime = System.currentTimeMillis(); 
      }

      synchronized protected long getLastUsedTime()
      {
         return lastUsedTime;
      }

      private long lastUsedTime = 0;
   }
}
