package com.jetbyte.cache8;

import java.util.Map;
import java.util.HashMap;
import java.util.LinkedList;

public class MaxTimeToLiveCache extends TimeLimitedCache
{
   public MaxTimeToLiveCache(String name)
   {
      super(name, MaxTimeToLiveCache.class.getName());
   }

   protected Object findObject(Cache.Key key)
   {      
      ObjectRef ref = (ObjectRef)cache.get(key);

      if (ref != null)
      {
         return ref.getObject();
      }

      return null;
   }

   protected void addObject(Cache.Key key, Object obj)
   {
      ObjectRef ref = new ObjectRef(key, obj); 

      cache.put(key, ref);

      creationOrderList.addLast(ref); 
   }

   protected void removeObject(Cache.Key key)
   {
      cache.remove(key);
   }

   synchronized public int entries()
   {
      return cache.size();
   }

   protected void flushCache()
   {
      cache.clear();

      creationOrderList.clear();
   }

   protected long getReapPeriod()
   {
      return scanEveryMillis;
   }

   synchronized protected void reap()
   {
      boolean done = (creationOrderList.size() == 0);
      
      System.out.println("Reaping - " + name());

      while (!done)
      {
         ObjectRef ref = (ObjectRef)creationOrderList.getFirst();

         if (ref.timedOut(entryTimeoutMillis))
         {
            System.out.println("Reference reaped: - " + ref.getKey().toString());
            
            cache.remove(ref.getKey());

            creationOrderList.removeFirst();

            done = (creationOrderList.size() == 0);
         }
         else
         {
            done = true;
         }
      }
   }

   private long entryTimeoutMillis = 30000;  // 30 secs
   
   private long scanEveryMillis = 10000; // 10 secs

   private Map cache = new HashMap(89);  
   
   private LinkedList creationOrderList = new LinkedList(); 

   private static class ObjectRef
   {
      protected ObjectRef(
         Cache.Key key, 
         Object obj)
      {
         this.key = key;
         this.obj = obj;

         creationTime = System.currentTimeMillis(); 
      }

      synchronized protected boolean timedOut(long timeoutMillis)
      {
         return (System.currentTimeMillis() - creationTime > timeoutMillis);
      }

      protected Cache.Key getKey()
      {
         return key;
      }

      public Object getObject()
      {
         return obj;
      }

      private final long creationTime;
      private final Object obj;
      private final Cache.Key key; 
   }

}
